@@ -14,7 +14,7 @@ from django.conf import settings
 #    Manager.get_query_set = Manager.get_queryset
 #except AttributeError:
 #    Manager.get_queryset = Manager.get_query_set
-
+from django.core.exceptions import ImproperlyConfigured
 
 if django.VERSION < (1, 8):
     from django.template import add_to_builtins
@@ -87,7 +87,7 @@ try:
     get_model = apps.get_model
 except ImportError:
     from django.db.models.loading import get_model
-    
+
 
 def get_model_name(model_cls):
     try:
@@ -305,18 +305,24 @@ if django.VERSION < (1, 5):
     from django.utils import simplejson
 else:
     import json as simplejson
-    
+
 try:
     from collections import OrderedDict as SortedDict
 except ImportError:
     from django.utils.datastructures import SortedDict
 
 
-# Django 1.7 compatibility
-try:
+# Backporting from 1.8
+if django.VERSION < (1, 8):
+    from compat.json_response import DjangoJSONEncoder
+else:
+    from django.core.serializers.json import DjangoJSONEncoder
+
+
+if django.VERSION < (1, 8):
+    from compat.json_response import JsonResponse
+else:
     from django.http import JsonResponse
-except:
-    from .json_response import JsonResponse
 
 
 # format_html (django 1.6)
@@ -351,6 +357,90 @@ except ImportError:  # django < 1.5
     from .shortcuts import resolve_url
 
 
+try:
+    from django.db import close_old_connections as close_connection
+except ImportError:  # django < 1.8
+    from django.db import close_connection
+
+
+def get_template_loaders():
+    """
+    Compatibility method to fetch the template loaders.
+    Source: https://github.com/django-debug-toolbar/django-debug-toolbar/blob/ece1c2775af108a92a0ef59636266b49e286e916/debug_toolbar/compat.py
+    """
+    try:
+        from django.template.engine import Engine
+    except ImportError:  # Django < 1.8
+        Engine = None
+
+    if Engine:
+        try:
+            engine = Engine.get_default()
+        except ImproperlyConfigured:
+            loaders = []
+        else:
+            loaders = engine.template_loaders
+    else:  # Django < 1.8
+        from django.template.loader import find_template_loader
+        loaders = [
+            find_template_loader(loader_name)
+            for loader_name in settings.TEMPLATE_LOADERS]
+    return loaders
+
+
+if django.VERSION >= (1, 10):
+    import django.urls as urlresolvers
+    from django.urls import (
+        clear_url_caches, get_script_prefix, get_urlconf,
+        is_valid_path, resolve, reverse, reverse_lazy, set_script_prefix,
+        set_urlconf, LocaleRegexProvider, LocaleRegexURLResolver, RegexURLPattern,
+        RegexURLResolver, ResolverMatch, get_ns_resolver, get_resolver, get_callable, get_mod_func
+    )
+else:
+    import django.core.urlresolvers as urlresolvers
+    from django.core.urlresolvers import (
+        clear_url_caches, get_script_prefix, get_urlconf,
+        is_valid_path, resolve, reverse, reverse_lazy, set_script_prefix,
+        set_urlconf, LocaleRegexProvider, LocaleRegexURLResolver, RegexURLPattern,
+        RegexURLResolver, ResolverMatch, get_ns_resolver, get_resolver, get_callable, get_mod_func
+    )
+
+
+from django.template.loader import render_to_string as render_to_string_django
+
+_context_instance_undefined = object()
+_dictionary_undefined = object()
+_dirs_undefined = object()
+
+def render_to_string(template_name, context=None,
+                     context_instance=_context_instance_undefined,
+                     dirs=_dirs_undefined,
+                     dictionary=_dictionary_undefined,
+                     request=None, using=None):
+    if (context_instance is _context_instance_undefined and dirs is _dirs_undefined and
+            dictionary is _dictionary_undefined):
+        if django.VERSION >= (1, 8):
+            # Call new render_to_string with new arguments
+            return render_to_string_django(template_name, context, request, using)
+        else:
+            # Call legacy render_to_string with new arguments
+            from django.template import RequestContext
+            context_instance = RequestContext(request) if request else None
+            return render_to_string_django(template_name, context, context_instance)
+    else:
+        if django.VERSION >= (1, 10):
+            # Call new render_to_string with legacy arguments
+            raise NotImplementedError('Django compat does not support calling post-1.8 render_to_string with pre-1.8 '
+                                      'keyword arguments')
+        else:
+            # Call legacy render_to_string with legacy arguments
+            if dictionary is _dictionary_undefined:
+                dictionary = {}
+            if context_instance is _context_instance_undefined:
+                context_instance = None
+            return render_to_string_django(template_name, dictionary, context_instance)
+
+
 ### Undocumented ###
 
 try:
@@ -374,6 +464,17 @@ else:
 # commit_on_success replaced by atomic in Django >=1.8
 atomic = commit_on_success = getattr(django.db.transaction, 'atomic', None) or getattr(django.db.transaction, 'commit_on_success')
 
+# Removed from django.contrib.sites.models in Django 1.9
+try:
+    from django.contrib.sites.shortcuts import get_current_site
+except ImportError:
+    from django.contrib.sites.models import get_current_site
+
+# Renamed utils and removed in Django 1.9
+try:
+    from django.contrib.admin import utils as admin_utils
+except ImportError:
+    from django.contrib.admin import util as admin_utils
 
 # the tests will try to import these
 __all__ = [
@@ -399,6 +500,7 @@ __all__ = [
     'urlunparse',
     'urlencode',
     'unquote_plus',
+    'DjangoJSONEncoder',
     'JsonResponse',
     'HttpResponseBase',
     'python_2_unicode_compatible',
@@ -420,4 +522,13 @@ __all__ = [
     'commit_on_success', # alias
     'format_html',
     'resolve_url',
+    'close_connection',
+    'get_template_loaders',
+    'LocaleRegexProvider', 'LocaleRegexURLResolver', 'NoReverseMatch', 'RegexURLPattern', 'RegexURLResolver',
+    'Resolver404', 'ResolverMatch', 'clear_url_caches', 'get_callable', 'get_mod_func', 'get_ns_resolver',
+    'get_resolver', 'get_script_prefix', 'get_urlconf', 'is_valid_path', 'resolve', 'reverse', 'reverse_lazy',
+    'set_script_prefix', 'set_urlconf',
+    'render_to_string',
+    'get_current_site',
+    'admin_utils'
 ]
